'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getJSONHash = exports.tagTime = exports.createServerLogger = undefined;

var _crypto = require('crypto');

var _Dr = require('dr-js/library/Dr.node');

const { File: { createDirectory }, Module: { createLogger } } = _Dr.Node;

const DEFAULT_LOG_LENGTH_THRESHOLD = 1024;
const FILE_SPLIT_INTERVAL = 24 * 60 * 60 * 1000;
const EMPTY_FUNC = () => {};
const createServerLogger = async ({
  pathLogDirectory,
  prefixLogFile = '',
  queueLengthThreshold = DEFAULT_LOG_LENGTH_THRESHOLD,
  fileSplitInterval = FILE_SPLIT_INTERVAL
}) => {
  let logger;
  if (!pathLogDirectory) {
    logger = { add: console.log, save: EMPTY_FUNC, split: EMPTY_FUNC, end: EMPTY_FUNC };
  } else {
    await createDirectory(pathLogDirectory);
    logger = createLogger({ pathLogDirectory, prefixLogFile, queueLengthThreshold, fileSplitInterval });
  }
  const { add } = logger;
  logger.add = text => add(`[${getTimeISO()}] ${text}`);
  return logger;
};

const getTimeISO = () => new Date().toISOString();

let time = Date.now();
const tagTime = message => {
  const currentTime = Date.now();
  const tagMessage = `[+${currentTime - time}ms] ${message}`;
  time = currentTime;
  return tagMessage;
};

const getJSONHash = object => (0, _crypto.createHash)('sha1').update(JSON.stringify(object), 'utf8').digest('base64').replace(/\W/g, '');

exports.createServerLogger = createServerLogger;
exports.tagTime = tagTime;
exports.getJSONHash = getJSONHash;