'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.configure = undefined;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _Dr = require('dr-js/library/Dr.node');

var _utils__ = require('./__utils__');

var _task = require('./task');

var _server = require('./server');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

const { System: { setProcessExitListener } } = _Dr.Node;

const TASK_AUTO_END_INTERVAL = 30 * 1000;

const configureTaskRunner = async ({
  exitListenerAsyncList, exitListenerList, serverLogger,
  urlSnapshotApp, urlServerPingTest, taskRunnerCount: clusterSize, taskRunnerDebugOnly
}) => {
  const logger = Object.assign({}, serverLogger, { add: log => serverLogger.add((0, _utils__.tagTime)(log)) });
  const puppeteerBrowser = taskRunnerDebugOnly ? undefined : await (0, _task.initPuppeteerBrowser)({ logger });
  puppeteerBrowser && puppeteerBrowser.on('disconnected', () => {
    logger.add('[Puppeteer] browser disconnected, exiting');
    process.exit();
  });
  const taskRunner = await (taskRunnerDebugOnly ? _task.createTaskRunnerDebug : _task.createTaskRunnerPuppeteer)({
    urlSnapshotApp, urlServerPingTest, puppeteerBrowser, clusterSize, logger
  });
  const taskEndToken = setInterval(() => taskRunner.autoEndTask(TASK_AUTO_END_INTERVAL * 0.001), TASK_AUTO_END_INTERVAL);
  exitListenerAsyncList.push(async () => {
    await taskRunner.clear();
    puppeteerBrowser && (await (0, _task.clearPuppeteerBrowser)({ puppeteerBrowser }));
  });
  exitListenerList.push(() => clearInterval(taskEndToken));
  return taskRunner;
};

const configure = async ({
  hostname, port,
  filePid, pathLog, logFilePrefix,
  urlServerPingTest, urlSnapshotApp, urlSnapshotTask,
  taskRunnerCount, taskRunnerDebugOnly = false
}) => {
  filePid && _fs2.default.writeFileSync(filePid, String(process.pid));

  const exitListenerAsyncList = [];
  const exitListenerList = [];
  setProcessExitListener({
    listenerAsync: async (_ref) => {
      let { eventType } = _ref,
          exitState = _objectWithoutProperties(_ref, ['eventType']);

      for (const listenerAsync of exitListenerAsyncList) await listenerAsync(eventType, exitState);
    },
    listenerSync: ({ eventType, code }) => {
      for (const listener of exitListenerList) listener(eventType, code);
      try {
        filePid && _fs2.default.unlinkSync(filePid);
      } catch (error) {}
    }
  });

  const serverLogger = await (0, _utils__.createServerLogger)({ pathLogDirectory: pathLog, prefixLogFile: logFilePrefix });
  exitListenerAsyncList.push((eventType, exitState) => serverLogger.add(`[SERVER DOWN] eventType: ${eventType}, exitState: ${JSON.stringify(exitState)}`));
  exitListenerList.push((eventType, code) => {
    serverLogger.add(`[SERVER EXIT] eventType: ${eventType}, code: ${code}`);
    serverLogger.end();
  });
  const taskRunner = await configureTaskRunner({ exitListenerAsyncList, exitListenerList, serverLogger, urlSnapshotApp, urlServerPingTest, taskRunnerCount, taskRunnerDebugOnly });
  const { start, option } = (0, _server.configureServer)({ hostname, port, serverLogger, urlSnapshotTask, taskRunner });

  start();

  serverLogger.add(`[SERVER UP] pid: ${process.pid}, running at: ${JSON.stringify(option)}`);
};

exports.configure = configure;