'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.exitWithError = exports.parseOption = undefined;

var _Dr = require('dr-js/library/Dr.node');

const { createOptionParser, OPTION_CONFIG_PRESET } = _Dr.Common.Module;
const {
  parseOptionMap,
  getOptionOptional, getSingleOptionOptional,
  getOption, getSingleOption
} = _Dr.Node.Module;

const CommonSingleStringPathFormat = Object.assign({}, OPTION_CONFIG_PRESET.SingleString, { isPath: true });

const OPTION_CONFIG = {
  prefixENV: 'imock-snapshot',
  formatList: [Object.assign({}, OPTION_CONFIG_PRESET.SingleString, {
    name: 'config',
    shortName: 'c',
    optional: true,
    description: `# from JSON: set to path relative process.cwd()\n# from ENV: set to 'env' to collect from process.env`
  }), Object.assign({}, OPTION_CONFIG_PRESET.SingleString, { name: 'hostname', shortName: 'h' }), Object.assign({}, OPTION_CONFIG_PRESET.SingleInteger, { name: 'port', shortName: 'p' }), Object.assign({}, OPTION_CONFIG_PRESET.SingleString, { name: 'url-server-ping-test' }), Object.assign({}, OPTION_CONFIG_PRESET.SingleString, { name: 'url-snapshot-app' }), Object.assign({}, OPTION_CONFIG_PRESET.SingleString, { name: 'url-snapshot-task' }), Object.assign({}, OPTION_CONFIG_PRESET.SingleInteger, { name: 'task-runner-count' }), Object.assign({}, OPTION_CONFIG_PRESET.SingleString, { name: 'task-runner-debug-only', optional: true }), Object.assign({}, CommonSingleStringPathFormat, { name: 'file-pid', optional: true }), Object.assign({}, CommonSingleStringPathFormat, {
    name: 'path-log',
    optional: true,
    extendFormatList: [Object.assign({}, OPTION_CONFIG_PRESET.SingleString, { name: 'prefix-log-file', optional: true })]
  })]
};

const { parseCLI, parseENV, parseJSON, processOptionMap, formatUsage } = createOptionParser(OPTION_CONFIG);

const parseOption = async () => ({
  optionMap: await parseOptionMap({ parseCLI, parseENV, parseJSON, processOptionMap }),
  getOption,
  getOptionOptional,
  getSingleOption,
  getSingleOptionOptional
});

const exitWithError = error => {
  console.warn(formatUsage(error.stack || error.toString()));
  process.exit(1);
};

exports.parseOption = parseOption;
exports.exitWithError = exitWithError;