'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.clearPuppeteerBrowser = exports.initPuppeteerBrowser = exports.getTaskRunnerOptionPuppeteer = undefined;

var _Dr = require('dr-js/library/Dr.node');

var _puppeteer = require('puppeteer');

var _puppeteer2 = _interopRequireDefault(_puppeteer);

var _snapshot = require('./snapshot');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const { Resource: { pingRequestAsync } } = _Dr.Node;
const DEFAULT_TIMEOUT_OPTION = { timeout: 10000 };

const getTaskRunnerOptionPuppeteer = ({ urlSnapshotApp, urlServerPingTest, puppeteerBrowser, logger }) => {
  let puppeteerPage = null;
  const resetRunner = (error, taskId) => {
    error && logger.add(`[Puppeteer|${taskId}] error: ${error.stack || error.toString()}`);
    return puppeteerPage && clearPuppeteerPage({ puppeteerPage }).then(() => {
      puppeteerPage = null;
    });
  };
  return {
    clearRunner: resetRunner,
    resetRunner,
    getTaskInitialState: task => Object.assign({}, task, { progress: 0 }),
    runTask: async (taskId, { getTaskState, updateTaskState }) => {
      logger.add(`[Puppeteer|${taskId}] start`);
      if (!puppeteerPage) puppeteerPage = await initPuppeteerPage({ puppeteerBrowser, logger });
      await pingRequestAsync({ url: urlServerPingTest, timeout: 2000, retryCount: 5 });
      if (!puppeteerPage.url().includes(urlSnapshotApp)) await setupPuppeteerPage({ urlSnapshotApp, puppeteerPage, logger });
      const { inputData } = getTaskState(taskId);

      const outputData = await (0, _snapshot.doSnapshot)({ inputData, puppeteerPage, logger, stepCallback: ({ progress }) => updateTaskState(taskId, { progress }) });
      logger.add(`[Puppeteer|${taskId}] complete`);
      return outputData;
    }
  };
};

const clearPuppeteerBrowser = ({ puppeteerBrowser }) => puppeteerBrowser.close().catch(() => {});

const initPuppeteerBrowser = async ({ logger }) => {
  const puppeteerBrowser = await _puppeteer2.default.launch({ headless: true, args: ['--no-sandbox'] });

  return puppeteerBrowser;
};

const clearPuppeteerPage = ({ puppeteerPage }) => puppeteerPage.close().catch(() => {});

const initPuppeteerPage = async ({ puppeteerBrowser, logger }) => {
  const puppeteerPage = await puppeteerBrowser.newPage();
  puppeteerPage.on('error', error => console.warn('[Puppeteer|Page] error:', error));
  puppeteerPage.on('pageerror', exceptionMessage => console.warn('[Puppeteer|Page] pageerror:', exceptionMessage));
  puppeteerPage.on('requestfailed', ({ url, method }) => console.warn('[Puppeteer|Page] requestfailed:', { url, method }));

  return puppeteerPage;
};

const setupPuppeteerPage = async ({ urlSnapshotApp, puppeteerPage, logger }) => {
  await puppeteerPage.goto(urlSnapshotApp, Object.assign({ waitUntil: 'networkidle2' }, DEFAULT_TIMEOUT_OPTION));
  await puppeteerPage.setUserAgent('Phantom WebKit');
  await puppeteerPage.setViewport({ width: 0, height: 0 });
  await puppeteerPage.waitForFunction(_snapshot.BROWSER_FUNCTION_MAP.checkReady, DEFAULT_TIMEOUT_OPTION);
  logger.add('[Puppeteer|Page] setup ready');
};

exports.getTaskRunnerOptionPuppeteer = getTaskRunnerOptionPuppeteer;
exports.initPuppeteerBrowser = initPuppeteerBrowser;
exports.clearPuppeteerBrowser = clearPuppeteerBrowser;