'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.doSnapshot = exports.BROWSER_FUNCTION_MAP = undefined;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const DEFAULT_TIMEOUT_OPTION = { timeout: 10000 };
const BROWSER_FUNCTION_MAP = {
  addSnapShotTask: ({ projectCid, canvasZoom, taskList }) => {
    window.SNAPSHOT_CONTROL.addSnapShotTask({ projectCid, canvasZoom, taskList });
  },
  checkReady: () => {
    const { isReady } = window.SNAPSHOT_CONTROL.getState();
    return isReady;
  },
  checkWait: () => {
    const { isReady, isWait } = window.SNAPSHOT_CONTROL.getState();
    return isReady && isWait;
  },
  getTaskResult: () => {
    const { state } = window.SNAPSHOT_CONTROL.getState();
    return state;
  },
  sendResume: () => {
    window.SNAPSHOT_CONTROL.resume();
  },
  sendReset: () => {
    window.SNAPSHOT_CONTROL.resetTaskQueue();
    window.SNAPSHOT_CONTROL.resume();
  }
};

const doSnapshot = async ({ puppeteerPage, inputData, stepCallback, logger }) => {
  const { projectCid, canvasZoom, taskList, pathOutput } = inputData;
  taskList.forEach(v => v.screenStateCid = v.screenStateCid || 'default');
  await puppeteerPage.waitForFunction(BROWSER_FUNCTION_MAP.checkReady, DEFAULT_TIMEOUT_OPTION);
  await puppeteerPage.evaluate(BROWSER_FUNCTION_MAP.addSnapShotTask, { projectCid, canvasZoom, taskList });

  const outputList = [];
  const indexMax = taskList.length;
  for (let index = 0; index < indexMax; index++) {
    await puppeteerPage.waitForFunction(BROWSER_FUNCTION_MAP.checkWait, DEFAULT_TIMEOUT_OPTION);

    const { canvasData, error } = await puppeteerPage.evaluate(BROWSER_FUNCTION_MAP.getTaskResult);
    if (error || !canvasData) {
      logger.add(`[snapshot|${index + 1}/${indexMax}] error: ${error}, canvasData: ${JSON.stringify(canvasData)}`);
      await puppeteerPage.evaluate(BROWSER_FUNCTION_MAP.sendReset);
      throw error;
    }

    const { screenCid, screenStateCid, canvasWidth, canvasHeight } = canvasData;
    const { fileName } = taskList.find(v => v.screenCid === screenCid && v.screenStateCid === screenStateCid);
    const filePath = _path2.default.join(pathOutput, fileName || `${screenCid}-${screenStateCid}_${canvasZoom}.png`);
    await puppeteerPage.screenshot({ path: filePath, clip: { x: 0, y: 0, width: canvasWidth, height: canvasHeight } });
    await puppeteerPage.evaluate(BROWSER_FUNCTION_MAP.sendResume);

    outputList.push({ screenCid, screenStateCid, canvasZoom, fileName, filePath });
    stepCallback({ progress: Math.round(100 * (index + 1) / indexMax) });
    logger.add(`[snapshot|${index + 1}/${indexMax}] file ready: ${filePath}`);
  }

  return outputList;
};

exports.BROWSER_FUNCTION_MAP = BROWSER_FUNCTION_MAP;
exports.doSnapshot = doSnapshot;